package org.lantsovanton.gameio;

import java.io.File;

import java.io.FileWriter;
import java.io.IOException;
import java.io.Writer;

import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;
import javax.xml.transform.Result;
import javax.xml.transform.Source;
import javax.xml.transform.Transformer;
import javax.xml.transform.TransformerConfigurationException;
import javax.xml.transform.TransformerException;
import javax.xml.transform.TransformerFactory;
import javax.xml.transform.TransformerFactoryConfigurationError;
import javax.xml.transform.dom.DOMSource;
import javax.xml.transform.stream.StreamResult;

import org.lantsovanton.GameException;
import org.lantsovanton.abstraction.model.IChip;
import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.Intellect;

import org.w3c.dom.Document;
import org.w3c.dom.Element;

public class XMLGameWriter implements IGameWriter{
	
	private IPosition myPosition;	
	private File myFile;
	private Document myDocument;
	private IGame myGame;
	private Intellect myIntellect;
	private boolean myBeautiful = true;
	
	public XMLGameWriter(){
		
	}
	
	public XMLGameWriter( IPosition position ){
		myPosition = position;
	}
	
	public void setPosition ( IPosition position ){
		myPosition = position;
	}
	
	public void setFile( File file ){
		myFile =  file;
	}
	
	public void save( String file ){
		myFile = new File( file );
		
			DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			myDocument = builder.newDocument();
			} catch ( ParserConfigurationException e ) {
			e.printStackTrace();
		}
		if( myPosition == null ){
			throw new GameException( " you try save unexisted position  " );
		}
		Element currentElement;
		Element counterElement;
		currentElement =  myDocument.createElement( "position" );
			currentElement.setAttribute("startplayer", "" + myPosition.getPlayer() );
		currentElement.setAttribute( "height", "" + myPosition.getHight() );
		currentElement.setAttribute( "width", "" + myPosition.getWidth() );
		currentElement.setAttribute( "game", myGame.getName() );
		currentElement.setAttribute( "intellect", myIntellect.getName() );
		if( ! myBeautiful )
		currentElement.setAttribute( "beautiful", "" + false );
		
		int h = myPosition.getHight();
		int l = myPosition.getWidth();
		
		for( int i=0; i < h; i++ ){
			for( int j = 0; j < l; j++ ){
				if( myPosition.get( i, j ) != null ){
					IChip counter = myPosition.get( i, j );
					counterElement = myDocument.createElement( "chip" );
					counterElement.setAttribute( "x", "" + j );
					counterElement.setAttribute( "y", "" + i );
					counterElement.setAttribute( "player", ""+counter.getPlayer());
					currentElement.appendChild( counterElement );
						}			
			}
		}
		
		try {
			myFile.createNewFile();
			Writer writer = new FileWriter( myFile );
			writer.write( myDocument.toString().toCharArray() );
			System.out.println( myDocument );
		} catch (IOException e) {			
			e.printStackTrace();
		}
		myDocument.appendChild( currentElement );
		Source source = new DOMSource( myDocument );
		Result result= new StreamResult( myFile );
		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.transform( source, result );
		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerFactoryConfigurationError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}

	public void setGame(IGame game) {
		myGame = game;		
	}

	public void setIntellect(Intellect intellect) {
		myIntellect = intellect;
	}

	public void save() {
		
		DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
		DocumentBuilder builder;
		try {
			builder = factory.newDocumentBuilder();
			myDocument = builder.newDocument();
			} catch ( ParserConfigurationException e ) {
			e.printStackTrace();
		}
		if( myPosition == null ){
			throw new GameException( " you try save unexisted position  " );
		}
		Element currentElement;
		Element counterElement;
		currentElement =  myDocument.createElement( "position" );
			currentElement.setAttribute("startplayer", "" + myPosition.getPlayer() );
		currentElement.setAttribute( "height", "" + myPosition.getHight() );
		currentElement.setAttribute( "width", "" + myPosition.getWidth() );
		currentElement.setAttribute( "game", myGame.getName() );
		currentElement.setAttribute( "intellect", myIntellect.getName() );
		currentElement.setAttribute( "beautiful", "" + myBeautiful);
		int h = myPosition.getHight();
		int l = myPosition.getWidth();
		
		for( int i=0; i < h; i++ ){
			for( int j = 0; j < l; j++ ){
				if( myPosition.get( i, j ) != null ){
					IChip counter = myPosition.get( i, j );
					counterElement = myDocument.createElement( "chip" );
					counterElement.setAttribute( "x", "" + j );
					counterElement.setAttribute( "y", "" + i );
					counterElement.setAttribute( "player", ""+counter.getPlayer());
					currentElement.appendChild( counterElement );
					}			
			}
		}
		
		try {
			myFile.createNewFile();
			Writer writer = new FileWriter( myFile );
			writer.write( myDocument.toString().toCharArray() );
			} catch (IOException e) {			
			e.printStackTrace();
		}
		myDocument.appendChild( currentElement );
		myDocument.normalize();
		Source source = new DOMSource( myDocument );
		myDocument.normalize();
		Result result= new StreamResult( myFile );
		try {
			Transformer transformer = TransformerFactory.newInstance().newTransformer();
			transformer.transform( source, result );
			
			
		} catch (TransformerConfigurationException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerFactoryConfigurationError e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (TransformerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
		
	}

	public void setBeautiful(boolean beautiful) {
		myBeautiful = beautiful;		
	}	
		
		
}


