package org.lantsovanton.games.attax;

import org.lantsovanton.GameException;
import org.lantsovanton.abstraction.model.Factory;
import org.lantsovanton.abstraction.model.ICoordinate;
import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IMove;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.Intellect;
//import org.lantsovanton.abstraction.model.impl.Coordinate;
//import org.lantsovanton.abstraction.model.impl.Chip;
//import org.lantsovanton.abstraction.model.impl.Position;
//import org.lantsovanton.abstraction.model.impl.PsevdoMove;
import org.lantsovanton.movers.AttaxMove;


public class Attax  implements IGame {
	
	private String myName = "Attax";
	// =true if any counter is choosen
	private boolean myState = false;
    private ICoordinate myCoordinate;
	private Factory myFactory = new Factory();
	
	public boolean isWinner(IPosition position, int player) {
		int n1 = 0;
		int n2 = 0;
		int nempty = 0;
		int numberofwin = 0; 
		for( int i = 0; i < 8; i++ ){
			for( int j = 0; j < 8; j++ ){
				if( position.get( i, j ) == null ){
					nempty++;
				}else{
					if( position.get( i, j ).getPlayer() == 1  ){
						n1++;
					}
					if( position.get( i, j ).getPlayer() == 2  ){
						n2++;
					}
				}
			}
		}
		if( n1 > n2 ){
			numberofwin = 1;
		}else{
			if( n2 > n1 ){
				numberofwin = 2;
			}
		}
		
		return numberofwin == player ;
	}

	public boolean isMovePosible(IPosition position, IMove move) {
		if( isGameFinish(position) ){
			return false;
		}
		
		if( ! ( move instanceof AttaxMove ) ){
			return false;
		}
		AttaxMove move1 = (AttaxMove)move;
		if( move1.getLength() != 2 ){
			return false;
		}
		if( move1.getPlayer() != position.getPlayer() ){
			return false;			
		}
		if( position.get( move1.get(0).getY(), move1.get(0).getX() ) == null ){
			return false;
		}
		if( position.get( move1.get(0).getY(), move1.get(0).getX() ).getPlayer() != position.getPlayer()  ){
			return false;
		}
		if( position.get( move1.get(1).getY(), move1.get(1).getX() ) != null ){
			return false;
		}
		int d1 = ( move1.get(0).getY() - move1.get(1).getY() ) * ( move1.get(0).getY() - move1.get(1).getY() ) ;
		int d2 = ( move1.get(0).getX() - move1.get(1).getX() ) * ( move1.get(0).getX() - move1.get(1).getX() );
		if( d1 > 4 || d2 > 4 ){
			return false;
		}
		
		return true;
	}

	public void doMove(IPosition position, IMove move) {
		if( ! isMovePosible( position,move ) ){
			throw new GameException();
		}
		AttaxMove move1 = (AttaxMove)move;
		int y0 = move1.get(0).getY();
		int x0 = move1.get(0).getX();
		
		int y = move1.get(1).getY();
		int x = move1.get(1).getX();
		position.set( y, x, myFactory.initialChip( position.getPlayer() ) );
		for( int i = -1; i < 2; i++ ){
			for( int j = -1; j < 2; j++ ){
				if( x +i < 8 && x+i > -1 && y+j < 8 && y + j > -1  ){
					if( position.get( y + j, x + i ) != null  ){
						position.set( y + j, x + i, myFactory.initialChip( position.getPlayer() ) );
					}
				}
			}
		}
		if( (x0-x)*(x0-x) + (y0-y)*(y0-y) > 2 ){
			position.set( y0, x0, null );
		}
		
		
		changePlayer(position);
		
		for( int h = 0; h < 8; h++ ){
			for( int l = 0; l < 8; l++ ){
				IPosition myPosition = position;
				int myPlayer = position.getPlayer();
				if( myPosition.get(h,l) != null ){
					if( myPosition.get(h,l).getPlayer() == myPlayer ){
						for ( int htemp = h - 2; htemp < h + 3; htemp++ ){
							for( int ltemp = l -2; ltemp < l + 3; ltemp++ ){
								
								if( htemp >= 0 && htemp < 8 && ltemp >= 0 && ltemp < 8 ){
									if( myPosition.get( htemp, ltemp ) == null ){
										return ;
									}
								}
							}
						}
					}
				}
				
			}
		}
		changePlayer(position);
	}
	
	private void changePlayer( IPosition posit ){
		if( posit.getPlayer() == 1 ){
			posit.setPlayer( 2 );
		}else{
			posit.setPlayer( 1 );
		}
	}

	public IPosition initialPosition() {
		IPosition position = myFactory.initialPosition( 8, 8, 1 );
		position.set( 0, 0, myFactory.initialChip(1) );
		position.set( 7, 7, myFactory.initialChip(1) );
		position.set( 0, 7, myFactory.initialChip(2) );
		position.set( 7, 0, myFactory.initialChip(2) );
		return position;
	}

	public boolean isGameFinish(IPosition position) {		
		int n1 = 0;
		int n2 = 0;
		int nempty = 0;
		for( int i = 0; i < 8; i++ ){
			for( int j = 0; j < 8; j++ ){
				if( position.get( i, j ) == null ){
					nempty++;
				}else{
					if( position.get( i, j ).getPlayer() == 1  ){
						n1++;
					}
					if( position.get( i, j ).getPlayer() == 2  ){
						n2++;
					}
				}
			}
		}
		if( n1 == 0 || n2 == 0 || nempty == 0  ){
			return true;
		}		
		
		return false;
	}

	public String whyMoveunPossible(IPosition position, IMove move) {
		
		return null;
	}

	public IMove getMove(IPosition position, ICoordinate coordinate) {
		if( ! myState  ){
			if(  position.get( coordinate.getY(), coordinate.getX() ) != null ){
				if( position.get( coordinate.getY(), coordinate.getX() ).getPlayer() == position.getPlayer()  ){
					myCoordinate =  coordinate;
					myState = true;
					return myFactory.initialPsevdoMove(1);
				}
			}else{
				myState = false;
				return null;
			}
			
		}else{ //myState =true
			AttaxMove move = new AttaxMove( position.getPlayer() );
			move.add( myCoordinate );
			move.add( coordinate );
			if( this.isMovePosible( position, move ) ){
				myState = false;
				return move; 
			}else{
				myState = false;
				return null;
				
			}
		}	
		myState = false;
		return null;
				
	}

	public String getName() {
		return myName;
	}

	public IPosition tryMove( IPosition posit, IMove m ) {
		IPosition result = (IPosition) posit.clone();
		doMove( result, m );
		return result;
	}

	public String getRegulation() {
		// TODO Auto-generated method stub
		return null;
	}

	public Intellect initialIntellect() {
		return new RandomAttax();
	}

}
