/**
 * 
 */
package org.lantsovanton.games.noughtsandcrosses;

import java.util.Random;

import org.lantsovanton.GameException;
import org.lantsovanton.abstraction.model.*;
import org.lantsovanton.abstraction.model.impl.*;


/**
 * @author 
 *
 */
public class Random001 implements Intellect {
	 private IGame myGame;
	 private int myPlayer;
	 private IPosition myPosition;
	 private String myName = "Random001";
	 private int myNumberofmove;
	 private boolean myPermit;
	 
	 public Random001(){		
		 myPermit = true;
	 }
	 
	 public Random001( IGame game, int player, IPosition position ){
		 myGame = game;
		 myPlayer = player;
		 myPosition = position;
	 }

	/** 
	 * @see org.lantsovanton.abstraction.model.Intellect#getGame()
	 */
	public IGame getGame() {
		return myGame;
	}

	/** 
	 * @see org.lantsovanton.intellect.Intellect#getPlayer()
	 */
	public int getPlayer() {		
		return myPlayer;
	}

	/** (non-Javadoc)
	 * @see org.lantsovanton.intellect.Intellect#getPosition()
	 */
	public IPosition getPosition() {		
		return myPosition;
	}


	/** (non-Javadoc)
	 * @see org.lantsovanton.intellect.Intellect#go()
	 */
	public IMove go() {
		
		if( myPosition.getPlayer() != myPlayer || myGame.isGameFinish( myPosition ) ){
			throw new GameException();
		}
		
		//
		myNumberofmove = 0;
		for( int h = 0; h <  myPosition.getHight(); h++  ){
			for( int l = 0; l < this.myPosition.getWidth(); l++ ){
				if( myPosition.get( h, l ) != null ){
					myNumberofmove++;
				}
			}
		}
		///
		
		
		for( int h = 0; h <  myPosition.getHight(); h++  ){
			for( int l = 0; l < this.myPosition.getWidth(); l++ ){
				IMove move = new Move( h, l, myPlayer );
				if( myGame.isMovePosible(myPosition, move ) )
				if( myGame.isGameFinish( myGame.tryMove( myPosition, move ) ) ){
					return move;
				}
			}
		}
		
		int otherPlayer = 1;
		if( myPlayer == 1 ){
			otherPlayer = 2; 
		}
		
		for( int h = 0; h <  myPosition.getHight(); h++  ){
			for( int l = 0; l < this.myPosition.getWidth(); l++ ){
				IPosition tempPosition = (IPosition)myPosition.clone();
				tempPosition.setPlayer( otherPlayer );
				IMove move = new Move( h, l, otherPlayer );
				if( myGame.isMovePosible( tempPosition, move ) )
				if( myGame.isGameFinish( myGame.tryMove( tempPosition, move ) ) ){
					return new Move( h,l, myPlayer );
				}
			}
		}		
		
		if( myNumberofmove == 0 ){
			IMove move = new Move( 1, 1, myPlayer );
			return move;
		}
		
		if( myNumberofmove == 1 ){
			if( myPosition.get(1,1) != null ){
				IMove move = new Move( 0, 0, myPlayer );
				return move;
			}else{
				IMove move = new Move( 1, 1, myPlayer );
				return move;
			}
		}
		
		if( myNumberofmove == 3 ){
			
			if( myPosition.get(1,1) != null ){
				if( myPosition.get(1,1).getPlayer() != myPlayer  ){
					IMove move = new Move( 0, 0, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
					move = new Move( 0, 2, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
					move = new Move( 2, 0, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
					move = new Move( 2, 2, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
				}else{
					IMove move = new Move( 0, 1, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
					move = new Move( 1, 0, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
					move = new Move( 2, 1, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
					move = new Move( 1, 2, myPlayer );
					if( myGame.isMovePosible( myPosition, move ) ){
						return move;
					}
				}
			}
		}		
		if( myPermit ){
			return goRandom();
		}else{
			return goFor();
		}
		
	}
	
	private IMove goRandom(){
		Random random = new Random(); 
		if( myPosition.getPlayer() != myPlayer || myGame.isGameFinish( myPosition ) ){
			throw new GameException();
		}
		while( true ){
			int h = random.nextInt( myPosition.getHight() );
			int l = random.nextInt( myPosition.getWidth() );
			IMove move = new Move( h, l, myPlayer );
			if( myGame.isMovePosible( myPosition, move ) ){
				return move;
			}
		}		
	}
	
	private IMove goFor(){
		for( int h=0; h < 3; h++  ){
			for( int l = 0; l < 3; l++ ){
				IMove move = new Move( h, l, myPlayer );
				if( myGame.isMovePosible( myPosition, move ) ){
					return move;
				}
			}
		}	
		return null;
	}

	public String getName() {		
		return myName;
	}

	public void setGame(IGame game) {
		myGame = game;
		
	}

	public void setPosition(IPosition position) {
		// TODO Auto-generated method stub
		myPosition = position;
	}

	public void setPlayer(int player) {
		// TODO Auto-generated method stub
		myPlayer = player;
	}

	public void setRandom(boolean permit) {
		myPermit = permit;
		
	}

}
