/**
 * 
 */
package org.lantsovanton.games.reversi;

import org.lantsovanton.GameException;
import org.lantsovanton.abstraction.model.Factory;
import org.lantsovanton.abstraction.model.ICoordinate;
import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IMove;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.Intellect;
import org.lantsovanton.abstraction.model.impl.Chip;
import org.lantsovanton.abstraction.model.impl.Move;
import org.lantsovanton.abstraction.model.impl.Position;



/**
 * @author 
 *
 */

public class Reversi implements IGame {
	private String myName = "Reversi";
    private Factory myFactory = new Factory();
	/* (non-Javadoc)
	 * @see org.lantsovanton.IGame#isWin(org.lantsovanton.Position, int)
	 */
	public boolean isWinner( IPosition posit, int player) {
		if(! isHaveNoMove( posit ) ){
			return false;
		}
		int n1 = 0;
		int n2 = 0;
		for( int i = 0; i < 8; i++ ){
			for( int j = 0; j < 8; j++ ){
				if( posit.get( i, j ) != null ){
					if( posit.get( i, j ).getPlayer() == 1 ){
						n1++;
					}else{
						n2++;
					}
				}
			}
		}
		if( player == 1 ){
			return n1 > n2;
		}else{
			return n2 > n1;
		}		
	}
	
	private boolean isHaveNoMove( IPosition position1 ){ 
		IPosition position = ( IPosition )position1.clone();
		for( int i = 0; i < 8; i++ ){
			for( int j = 0; j < 8; j++ ){
				IMove m = myFactory.initialSimpleMove( i, j, 1 );
				position.setPlayer(1);
				if( isMovePosible(position, m) ){
					return false;
				}
				m = myFactory.initialSimpleMove( i, j, 2 );
				position.setPlayer(2);
				if( isMovePosible(position, m) ){
					return false;
				}
			}
		}		
		return true;
	}

	/* (non-Javadoc)
	 * @see org.lantsovanton.IGame#isPosible(org.lantsovanton.Position, org.lantsovanton.Move)
	 * do not ready
	 */
	public boolean isMovePosible( IPosition posit, IMove m1 ) {
		if( ! ( m1 instanceof Move ) ){
			return false;
		}
		Move m = ( Move )m1; 
		
		if( m.x > 7 || m.x < 0 || m.y < 0 || m.y > 7 ){
			return false;
		}
		
		if( posit.get( m.y, m.x ) != null ){
			return false;
		}
		
		if( m.player != posit.getPlayer() ){
			return false;
		}
		
		
		
		////////////////////////////////////
//		may be error later
		for( int h = m.y + 1; h < 8; h++ ){	
		    if( posit.get( h, m.x ) != null ){
	    		if( posit.get( h, m.x ).getPlayer() == m.player ){
	    			for( int i = m.y + 1; i < h; i++ ){
	    				return true;
	    			}
	    			break;
	    		}	
		    }else{
		    	break;
		    }
	    }
		
		 //may be error later
		for( int h = m.y - 1; h >= 0; h-- ){		    	
	    	if( posit.get( h, m.x ) != null ){	
			if( posit.get( h, m.x ).getPlayer() == m.player ){
	    			for( int i = m.y - 1; i > h; i-- ){
	    				return true;
	    			}
	    			break;
	    		}		    	
	    	}else{
	    		break;
	    	}
	    }
		
		 //may be error later
		for( int l = m.x + 1; l < 8; l++ ){		    	
	    	if( posit.get( m.y, l ) != null ){	
			if( posit.get( m.y, l ).getPlayer() == m.player ){
	    			for( int i = m.x + 1; i < l; i++ ){
	    				return true;
	    			}
	    			break;
	    		}		
	    	}else{
	    		break;
	    	}
	    }
		
		 //may be error later
		for( int l = m.x - 1; l >= 0; l-- ){		    	
	    	if( posit.get( m.y, l ) != null ){	
			if( posit.get( m.y, l ).getPlayer() == m.player ){
	    			for( int i = m.x - 1; i > l; i-- ){
	    				return true;
	    			}
	    			break;
	    		}
	    	}else{
	    		break;
	    	}
	    }
		
		///////////////////////////////////
		
		for( int i = 1; i + m.y < 8  && i + m.x < 8; i++ ){
			if( posit.get ( m.y + i, m.x + i ) != null ){
				if( posit.get( m.y + i, m.x + i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						return true;
					}
					break;
				}					
			}else{
				break;
			}				
		}
		
		for( int i = 1; m.y - i >= 0 && m.x - i >= 0; i++ ){
			if(posit.get( m.y - i, m.x - i ) != null ){
				if( posit.get( m.y - i, m.x - i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						return true;
					}
					break;
				}
			}else{
				break;
			}
		}
		////////////////////////////////
		for( int i = 1; m.y - i >= 0 && m.x + i < 8; i++ ){
			if(posit.get( m.y - i, m.x + i ) != null ){
				if( posit.get( m.y - i, m.x + i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						return true;
					}
					break;
				}
			}else{
				break;
			}
		}
		
		for( int i = 1; m.y + i < 8 && m.x - i >= 0; i++ ){
			if(posit.get( m.y + i, m.x - i ) != null ){
				if( posit.get( m.y + i, m.x - i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						return true;
					}
					break;
				}
			}else{
				break;
			}
		}
		////////////////////////////////
		return false;
	}

	/* (non-Javadoc)
	 * @see org.lantsovanton.IGame#makeMove(org.lantsovanton.Position, org.lantsovanton.Move)
	 * do not ready
	 */
	public void doMove( IPosition posit, IMove m1 ) {
		if( ! isMovePosible( posit, m1 ) ){		
			throw new GameException();			
		}
		if(!( m1 instanceof Move ) ){
			throw new GameException();
		}
		Move m = ( Move )m1;
		
		posit.set( m.y, m.x, new Chip( m.player ) );
	    
		//may be error later
		for( int h = m.y + 1; h < 8; h++ ){	
		    if( posit.get( h, m.x ) != null ){
	    		if( posit.get( h, m.x ).getPlayer() == m.player ){
	    			for( int i = m.y + 1; i < h; i++ ){
	    				posit.set( i, m.x, new Chip( m.player ) );
	    			}
	    			break;
	    		}	
		    }else{
		    	break;
		    }
	    }
		
		 //may be error later
		for( int h = m.y - 1; h >= 0; h-- ){		    	
	    	if( posit.get( h, m.x ) != null ){	
			if( posit.get( h, m.x ).getPlayer() == m.player ){
	    			for( int i = m.y - 1; i > h; i-- ){
	    				posit.set( i, m.x, new Chip( m.player ) );
	    			}
	    			break;
	    		}		    	
	    	}else{
	    		break;
	    	}
	    }
		
		 //may be error later
		for( int l = m.x + 1; l < 8; l++ ){		    	
	    	if( posit.get( m.y, l ) != null ){	
			if( posit.get( m.y, l ).getPlayer() == m.player ){
	    			for( int i = m.x + 1; i < l; i++ ){
	    				posit.set( m.y, i, new Chip( m.player ) );
	    			}
	    			break;
	    		}		
	    	}else{
	    		break;
	    	}
	    }
		
		 //may be error later
		for( int l = m.x - 1; l >= 0; l-- ){		    	
	    	if( posit.get( m.y, l ) != null ){	
			if( posit.get( m.y, l ).getPlayer() == m.player ){
	    			for( int i = m.x - 1; i > l; i-- ){
	    				posit.set( m.y, i, new Chip( m.player ) );
	    			}
	    			break;
	    		}
	    	}else{
	    		break;
	    	}
	    }
		//set diagonal elements later			
		
		for( int i = 1; i + m.y < 8  && i + m.x < 8; i++ ){
			if( posit.get ( m.y + i, m.x + i ) != null ){
				if( posit.get( m.y + i, m.x + i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						posit.set( m.y + j, m.x + j, new Chip(  m.player ) );
					}
					break;
				}					
			}else{
				break;
			}				
		}
		
		for( int i = 1; m.y - i >= 0 && m.x - i >= 0; i++ ){
			if(posit.get( m.y - i, m.x - i ) != null ){
				if( posit.get( m.y - i, m.x - i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						posit.set( m.y - j, m.x - j, new Chip(  m.player ) );
					}
					break;
				}
			}else{
				break;
			}
		}
		
		//////////////////////////////
		for( int i = 1; m.y - i >= 0 && m.x + i < 8; i++ ){
			if(posit.get( m.y - i, m.x + i ) != null ){
				if( posit.get( m.y - i, m.x + i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						posit.set( m.y - j, m.x + j, new Chip( m.player ) );
					}
					break;
				}
			}else{
				break;
			}
		}
		
		for( int i = 1; m.y + i < 8 && m.x - i >= 0; i++ ){
			if(posit.get( m.y + i, m.x - i ) != null ){
				if( posit.get( m.y + i, m.x - i ).getPlayer() == m.player ){
					for( int j = 1; j < i; j++ ){
						posit.set( m.y + j, m.x - j, new Chip( m.player ) );
					}
					break;
				}
			}else{
				break;
			}
		}			
		
		///end set  diagonal elements
		
		changePlayer( posit );
		if( isGameFinish( posit ) ){
			changePlayer( posit );
		}
	}
	
	private void changePlayer( IPosition posit ){
		if( posit.getPlayer() == 1 ){
			posit.setPlayer( 2 );
		}else{
			posit.setPlayer( 1 );
		}
	}

	/* (non-Javadoc)
	 * @see org.lantsovanton.IGame#startGame()
	 */
	public IPosition initialPosition() {
		Position result = new  Position( 8, 8, 1 );
		result.set( 3, 3, new Chip( 1 ) );
		result.set( 4, 4, new Chip( 1 ) );
		result.set( 4, 3, new Chip( 2 ) );
		result.set( 3, 4, new Chip( 2 ) );
		return result;
	}

	/** (non-Javadoc)
	 * not fast metod
	 * @see org.lantsovanton.games.IGame#isEnd(org.lantsovanton.abstraction.model.impl.Position)
	 */	
	public boolean isGameFinish( IPosition posit) {
		for( int i = 0; i < 8; i++ ){
			for( int j = 0; j < 8; j++ ){
				Move m = new Move( i, j, posit.getPlayer() );
				if( isMovePosible(posit, m) ){
					return false;
				}
			}
		}		
		return true;
	}

	public IPosition tryMove( IPosition posit, IMove m ) {
		Position result = ( Position ) posit.clone();
		doMove( result, m );
		return result;
	}

	public String getRegulation() {
		// TODO Auto-generated method stub
		return null;
	}

	public String whyMoveunPossible( IPosition position, IMove move) {
		if( !( move instanceof Move ) ){
			return "        ";
		}
		Move m = (Move)move;
		if( isGameFinish(position ) ){
			return "  .       .  ";
		}
		if( position.getPlayer() != m.player ){
			return "     ";
		}
		if( position.get(m.y, m.x) != null ){ 
			return "           ";
		}
		if( this.isMovePosible(position, move) ){ 
			return "  ";
		}else{
			return "         "; 
		}
		
		
	}

	public IMove getMove(IPosition position, ICoordinate coordinate) {
		Move move = new Move( coordinate.getY(), coordinate.getX(),  position.getPlayer() );
		if( isMovePosible(position,move) ){
			return move;
		}
		return null;
	}

	public String getName() {
		return myName;
	}

	public Intellect initialIntellect() {
		return new RandomReversi();
	}

	
	

}
