package test.org.lantsovanton;

import org.lantsovanton.abstraction.model.IGame;
import org.lantsovanton.abstraction.model.IPosition;
import org.lantsovanton.abstraction.model.Intellect;
import org.lantsovanton.abstraction.model.impl.Chip;
import org.lantsovanton.abstraction.model.impl.Move;
import org.lantsovanton.games.reversi.RandomReversi;
import org.lantsovanton.games.reversi.Reversi;

import junit.framework.TestCase;

public class RandomReversiTest extends TestCase {
	public void testnotrandom(){
		
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		Intellect intellect = new RandomReversi();
		intellect.setPlayer(1);
		intellect.setGame(game);
		intellect.setPosition(position);
		intellect.setRandom(false);
		Move move = new Move(2,4,1);
		assertEquals( intellect.go(), move );		
		
	}
	
	public void testcorner(){
		
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		Intellect intellect = new RandomReversi();
		intellect.setPlayer(1);
		intellect.setGame(game);
		intellect.setPosition(position);
		intellect.setRandom(false);
		
		for(int h=0;h<8;h++ ){
			for(int l=0;l<8;l++ ){
				position.set( h, l, new Chip( 2 ) );
			}
		}
		
		for(int l=0;l<8;l++ ){
			int h = 0;
			position.set( h, l, null );
		}
		
		position.set( 3,3, new Chip( 1 ) );
		Move move = new Move(0,0,1);
		assertEquals( intellect.go(), move );		
		
	}
	
	public void testline(){
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		Intellect intellect = new RandomReversi();
		intellect.setPlayer(1);
		intellect.setGame(game);
		intellect.setPosition(position);
		intellect.setRandom(false);
		
		for(int h=0;h<8;h++ ){
			for(int l=0;l<8;l++ ){
				position.set( h, l, null );
			}
		}
		
		for( int h=4;h<6;h++ ){
			for( int l=1;l<6;l++ ){
				position.set( h, l, new Chip( 2 ) );
			}
		}
		
		position.set( 5, 4, new Chip( 1 ) );
		Move move = new Move( 5, 0, 1 );
		assertEquals( intellect.go(), move );				
		
	}
	
	public void testlinedifficult(){
		
		IGame game = new Reversi();
		IPosition position = game.initialPosition();
		Intellect intellect = new RandomReversi();
		intellect.setPlayer(1);
		intellect.setGame(game);
		intellect.setPosition(position);
		intellect.setRandom(false);
		position.setPlayer(1);
		
		for(int h=0;h<8;h++ ){
			for(int l=0;l<8;l++ ){
				position.set( h, l, null );
			}
		}
		
		for( int h=1;h<7;h++ ){
			for( int l=1;l<7;l++ ){
				position.set( h, l, new Chip( 2 ) );
			}
		}
		
		position.set( 2,0, new Chip( 2 ) );
		position.set( 3,2, new Chip( 1 ) );
		
		Move move = new Move( 0, 2, 1 );
		assertEquals( intellect.go(), move );
	}
	

}
